//+------------------------------------------------------------------+
//|                                                      PrimaEA.mq4 |
//|                                                Maxim A.Kuznetsov |
//|                                                       nektomk.ru |
//+------------------------------------------------------------------+
#property copyright "Maxim A.Kuznetsov"
#property link      "nektomk.ru"
#property version   "1.00"
#property strict
//--- input parameters
sinput ENUM_TIMEFRAMES     TF=0;
sinput string  PRIMA="prima\\prima";
input int      PERIOD1=34;
input int      PERIOD2=200;
input int      DISTANCE1=75;
input int      DISTANCE2=150;
input int      SLIPPAGE=5;
input double   LOTS=0.01;
input int      SL=750;
input int      TP=900;
input int      TTL=0;
input ENUM_DAY_OF_WEEK      WEEKDAY_FROM=MONDAY;
input int      WEEKDAYS=7;
input int      HOUR_FROM=0;
input int            HOURS=24;
//input int      _HOUR_TO=24;
input bool     LEAVE_MARKET=true;
sinput int     MAGIK=2016071601;

#define MAX_OPEN_ATTEMPTS 5
#define MAX_CLOSE_ATTEMPTS 5

#include <stdlib.mqh>
#include "ea.mqh"

class PrimaEA: public EA {
public:
   PrimaEA(ENUM_TIMEFRAMES,string name,int magik);
   ~PrimaEA();   
   int Setup(int _period1,int _period2,int _distance1,int _distance2,string _primaPath);
   void OnConnect();
   void EveryBar();
   int Buy();
   int Sell();
public:   
   PrimaInd *prima;
};
PrimaEA::PrimaEA(ENUM_TIMEFRAMES _tf, string _name,int _magik):EA(_tf,_name,_magik)
{
   prima=NULL;
}
PrimaEA::~PrimaEA()
{
   if (prima!=NULL) delete prima;
}
int PrimaEA::Setup(int _period1,int _period2,int _distance1,int _distance2,string _primaPath)
{
   if (_period1<2 || _period2<2 || _period1==_period2) return INIT_PARAMETERS_INCORRECT;
   prima=new PrimaInd(_period1,_period2,_distance1,_distance2,_primaPath);
   if (!prima.IsCorrect()) return INIT_PARAMETERS_INCORRECT;
   return INIT_SUCCEEDED;
}
void PrimaEA::EveryBar(void) {
   EA::EveryBar();
   if (prima.SigBuy()!=EMPTY_VALUE) {
      RequestBuy();
   } else if (prima.SigSell()!=EMPTY_VALUE) {
      RequestSell();
   }
}
int PrimaEA::Buy() {
   CloseAll(OP_SELL);
   return EA::Buy();
}
int PrimaEA::Sell() {
   CloseAll(OP_BUY);
   return EA::Sell();
}
void PrimaEA::OnConnect() {
   int bar; //      
   int op;  //   
   double priceMin,priceMax;
   EA::OnConnect();   // overloaded
   /// draw past 30 signals
   int t,n;
   n=0;
   for(t=0;t<iBars(_Symbol,tf);t++) {
      switch(prima.Op(t)) {
         case OP_BUY: DrawSignal(OP_BUY,iTime(_Symbol,tf,t),iOpen(_Symbol,tf,t));n++;break;
         case OP_SELL: DrawSignal(OP_SELL,iTime(_Symbol,tf,t),iOpen(_Symbol,tf,t));n++;break;
      }
      if (n==30) break;
   }
   // check omitted
   bar=prima.Prev();
   if (bar>=0 && (bar<PERIOD1 || bar<PERIOD2)) {
      //    
      op=prima.Op(bar);
      if (op==OP_BUY) {
         CloseAll(OP_SELL);
      } else if (op==OP_SELL) {
         CloseAll(OP_BUY);
      }
      priceMax=iOpen(_Symbol,prima.tf,op)+MathMin(DISTANCE1,DISTANCE2)*_Point+SLIPPAGE*_Point;
      priceMin=iOpen(_Symbol,prima.tf,op)-MathMin(DISTANCE1,DISTANCE2)*_Point-SLIPPAGE*_Point;
      if (Bid<priceMax && Bid>priceMin) {
         //     
         //       
         int pos,total;
         total=OrdersTotal();
         for(pos=total-1;pos>=0;pos--) {
            if (!OrderSelect(pos,SELECT_BY_POS,MODE_TRADES)) continue;
            if (OrderSymbol()!=_Symbol || OrderMagicNumber()!=MAGIK) continue;
            if (OrderCloseTime()!=0) continue;
            if (OrderType()!=op) continue;
            //    -   ,  
            //if (OrderOpenTime()<iTime(_Symbol,prima.tf,bar) ) continue;
            if (OrderOpenPrice()>priceMin && OrderOpenPrice()<priceMax) {
               //     - 
               return;
            }
         }
         //     
         //   
         if (op==OP_BUY) Buy();
         else if (op==OP_SELL) Sell();
      } 
   }
}
bool OpenAllowedByShed(datetime time,int hourFrom=0,int hourTo=24,int weekdayFrom=0,int weekdayTo=7) {
   int weekday;
   int hour;
   hour=TimeHour(time);
   if (hourTo>hourFrom) {
      if (hour<hourFrom || hour>=hourTo) return false;
   } else {
      if (hour>=hourTo && hour<hourFrom) return false;
   }
   weekday=TimeDayOfWeek(time);
   if (weekdayFrom<weekdayTo) {
      if (weekday<weekdayFrom || weekday>weekdayTo) return false;
   } else {
      if (weekday>weekdayTo && weekday<weekdayFrom) return false;
   }
   return true; 
}
 
//////////////////////
PrimaEA *ea=NULL;
int OnInit()
{
   ea=new PrimaEA(TF,"PrimaEA",MAGIK);
   ea.SetTradeTime(WEEKDAY_FROM,WEEKDAYS,HOUR_FROM,HOURS,1);
   ea.Setup(PERIOD1,PERIOD2,DISTANCE1,DISTANCE2,"prima\\prima");
   ea.lots=LOTS;
   ea.stopLoss=SL;
   ea.takeProfit=TP;
   ea.slippage=SLIPPAGE;
   ea.ttl=TTL;
   if (!ea.IsCorrect()) {
      return INIT_PARAMETERS_INCORRECT;
   }
   EventSetTimer(10);
   return(INIT_SUCCEEDED);
}
void OnDeinit(const int reason)
{
   if (ea!=NULL) delete ea;
}
void OnTimer()
{
   ea.OnTimer();
}
void OnTick()
{
   ea.EveryTick();
   return;
}
void OnChartEvent(const int id,         //     
                  const long& lparam,   //    long 
                  const double& dparam, //    double 
                  const string& sparam  //    string 
  ) {
  if (ea.OnEvent(id,lparam,dparam,sparam)) return;
  return;
}  